/* OV511 Decompression Support Module (No-MMX version)
 *
 * Copyright (c) 1999-2001 Mark W. McClelland
 * Original decompression code Copyright 1998-2000 OmniVision Technologies
 *
 * Please see the file: linux/Documentation/usb/ov511.txt 
 * and the website at:  http://alpha.dyndns.org/ov511
 * for more info.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/init.h>

#include "ov511.h"

extern int ov511_register_decomp_module(int ver, struct ov51x_decomp_ops *ops, int ov518, int mmx);
extern void ov511_deregister_decomp_module(int ov518, int mmx);
extern void * ov511_vmalloc(unsigned long size);
extern void ov511_vfree(void * addr);

/* Large memory buffer allocation. */
#define ALLOCATE(a, size, b) ov511_vmalloc(size)
#define FREE(addr) ov511_vfree(addr)

static const char version[] = "1.0";
static const int interface_ver = DECOMP_INTERFACE_VER;

static const int ov518 = 0;
static const int mmx = 0;

/* 0=no debug messages
 * 1=init/detection/unload and other significant messages,
 * 2=some warning messages
 * 3=config/control function calls
 * 4=most function calls and data parsing messages
 * 5=highly repetitive mesgs
 * NOTE: This should be changed to 0, 1, or 2 for production kernels
 */
static int debug = 0;

MODULE_PARM(debug, "i");
MODULE_PARM_DESC(debug, "Debug level: 0=none, 1=init/detection, 2=warning, 3=config/control, 4=function call, 5=max");

MODULE_AUTHOR("Mark McClelland <mmcclell@bigfoot.com>, OmniVision Technologies (http://www.ovt.com/)");
MODULE_DESCRIPTION("OV511 Decompression Module");

/* Tables */
static UCHAR YQuanTable[] = YQUANTABLE;
static UCHAR UVQuanTable[] = UVQUANTABLE;
static SHORT IDCTTable[] = IDCTTABLE;
static ULONG ZigZagTable[] = ZIGZAGTABLE;

static PUCHAR GetYQuanTable(void)
{
	return YQuanTable;
}

static PUCHAR GetUVQuanTable(void)
{
	return UVQuanTable;
}

static PSHORT GetIDCTTable(void)
{
	return IDCTTable;
}

static PULONG GetZigZagTable(void)
{
	return ZigZagTable;
}

static int DecompressYHINoMMX(	LPBYTE    lpDestBuf,
				LPBYTE    lpTMPBUF,
				ULONG     index,
				DWORD     IPTRR,
				DWORD     X_WIDE,
				PUCHAR    TableBufferYUV,
				PSHORT    IDCT_Coef,
				PULONG    ZZLN,
				DWORDLONG UV_Scalor,
				ULONG     YUVFlag,
				CompInfo *TempInfo)
{
	short ZigZag[64];
	int temp[64];
	int Zcnt_Flag=0;
	int Num8_Flag=0;
	int tmp, tmp1, tmp2, tmp3;
	unsigned char header, ZTable[64];
	short tmpl, tmph, half_byte, idx, count;
	DWORD ZigZag_length=0, ZT_length, i, j;
//	DWORD Start, k;
	short DeZigZag[64]; 

	const short a = 11584;
	const short b = 16068;
	const short c = 15136;
	const short d = 13624;
	const short e =  9104;
	const short f =  6270;
	const short g =  3196;

	int	IPTRR_YUV;

#if 0
	int	 coef[64] = 
	{
		11584,  11584,  11584,  11584,  11584,  11584,  11584,  11584,
		16068,  13624,   9104,   3196,  -3196,  -9104, -13624, -16068,
		15136,   6270,  -6270, -15136, -15136,  -6270,   6270,  15136,
		13624,  -3196, -16068,  -9104,   9104,  16068,   3196, -13624,
		11584, -11584, -11584,  11584,  11584, -11584, -11584,  11584,
		 9104, -16068,   3196,  13624, -13624,  -3196,  16068,  -9104,
		 6270, -15136,  15136,  -6270,  -6270,  15136, -15136,   6270,
		 3196,  -9104,  13624, -16068,  16068, -13624,   9104,  -3196
	};

	int	 coeft[64] = 
	{
		11584,  16068,  15136,  13624,  11584,   9104,   6270,   3196, 
		11584,  13624,   6270,  -3196, -11584, -16068, -15136,  -9104, 
		11584,   9104,  -6270, -16068, -11584,   3196,  15136,  13624, 
		11584,   3196, -15136,  -9104,  11584,  13624,  -6270, -16068, 
		11584,  -3196, -15136,   9104,  11584, -13624,  -6270,  16068, 
		11584,  -9104,  -6270,  16068, -11584,  -3196,  15136, -13624, 
		11584, -13624,   6270,   3196, -11584,  16068, -15136,   9104, 
		11584, -16068,  15136, -13624,  11584,  -9104,   6270,  -3196
	};
#endif

	// Take off every 'Zig'
	for(i=0; i<64; i++)
	{
		ZigZag[i] = 0;
	}

// Read in the Y header byte  

	header = lpTMPBUF[index];
	index++;

	ZigZag_length = header & 0x3f;
	ZigZag_length = ZigZag_length + 1;

	Num8_Flag = header & 0x40;
	Zcnt_Flag = header & 0x80;

// Read in the Y content
#if 1
	if(Zcnt_Flag==0)  // Without Zero Table Read contents directly
	{
		// Read in ZigZag[0]
		ZigZag[0] = lpTMPBUF[index++];
		tmpl = lpTMPBUF[index++];
		tmph = tmpl<<8;
		ZigZag[0] = ZigZag[0] | tmph;
		ZigZag[0] = ZigZag[0]<<4;
		ZigZag[0] = ZigZag[0]>>4;

		if(Num8_Flag)  // 8 Bits
		{
			for(i=1; i<ZigZag_length; i++)
			{
				ZigZag[i] = lpTMPBUF[index++];
				ZigZag[i] = ZigZag[i]<<8;
				ZigZag[i] = ZigZag[i]>>8;

			}
		}
		else     // 12 bits and has no Zero table
		{
				idx=1;
				half_byte=0;
				for(i=1; i<ZigZag_length; i++)
				{
					if(half_byte==0)
					{
						ZigZag[i] = lpTMPBUF[index++];
						tmpl = lpTMPBUF[index++];
						tmph = tmpl<<8;
						tmph = tmph&0x0f00;
						ZigZag[i] = ZigZag[i] | tmph;
						ZigZag[i] = ZigZag[i]<<4;
						ZigZag[i] = ZigZag[i]>>4;
						half_byte=1;
					}
					else
					{
						ZigZag[i] = lpTMPBUF[index++];
						ZigZag[i] = ZigZag[i]<<8;
						tmpl = tmpl & 0x00f0;
						ZigZag[i] = ZigZag[i] | tmpl;
						ZigZag[i] = ZigZag[i]>>4;
						half_byte=0;
					}
				}

		}	// end of Num8_Flag=0
	}	//  end of without Zero Table
	else  // Has Zero Table
	{
			// calculate Z-Table length
			ZT_length = ZigZag_length/8;
			tmp = ZigZag_length%8;
			if(tmp> 0)
			{
				ZT_length = ZT_length + 1;
			}

			// read in Zero Table
			for(j=0; j<ZT_length; j++)
			{
				ZTable[j]=lpTMPBUF[index++];
			}

			// Read in ZigZag[0]
			ZigZag[0] = lpTMPBUF[index++];
			tmpl = lpTMPBUF[index++];
			tmph = tmpl<<8;
			ZigZag[0] = ZigZag[0] | tmph;
			ZigZag[0] = ZigZag[0]<<4;
			ZigZag[0] = ZigZag[0]>>4;

			// Decode ZiaZag
			idx=0;
			ZTable[idx]=ZTable[idx]<<1;
			count=7;

		if(Num8_Flag)  // 8 Bits and has zero table
		{
			for( i=1; i<ZigZag_length; i++)
			{
//				tmp=(ZTable[idx]&0x80);
//				if(tmp)
				if((ZTable[idx]&0x80))
				{
					ZigZag[i] = lpTMPBUF[index++];
					ZigZag[i] = ZigZag[i]<<8;
					ZigZag[i] = ZigZag[i]>>8;
				}

				ZTable[idx]=ZTable[idx]<<1;
				count--;
				if(count==0)
				{
					count=8;
					idx++;
				}
			}

		}// end of 8 bits with zero table
		else   // 12bits has zero table
		{
			half_byte=0;
			for( i=1; i<ZigZag_length; i++)
			{

				if(ZTable[idx]&0x80)
				{
					if(half_byte==0)
					{
						ZigZag[i] = lpTMPBUF[index++];
						tmpl = lpTMPBUF[index++];
						tmph = tmpl <<8;
						tmph = tmph & 0x0f00;
						ZigZag[i] = ZigZag[i] | tmph;
						ZigZag[i] = ZigZag[i]<<4;
						ZigZag[i] = ZigZag[i]>>4;
						half_byte=1;
					}
					else
					{
						ZigZag[i] = lpTMPBUF[index++];
						ZigZag[i] = ZigZag[i]<<8;
						tmpl = tmpl & 0x00f0;
						ZigZag[i] = ZigZag[i] | tmpl;
						ZigZag[i] = ZigZag[i]>>4;
						half_byte=0;
					}
				}
				
				ZTable[idx]=ZTable[idx]<<1;
				count--;
				if(count==0)
				{
					count=8;
					idx++;
				}
			}
		}// end of 12 bits with zero table	
	}		
#endif 


#if 0
	//DeZigZag
	idx=0;
	for(i=0; i<8; i++)
	{
		for(j=0; j<8; j++)
		{
			DeZigZag[idx] = ZigZag[ZZTB[idx]];
			idx++;
		}
	}
#endif
#if 1
	for(j=0; j<64; j++)
	{
		DeZigZag[j] = 0;
	}

	if(YUVFlag == 1)
	{
	DeZigZag[0] = ZigZag[0];
	DeZigZag[1] = ZigZag[1]<<1;
	DeZigZag[2] = ZigZag[5]<<1;
	DeZigZag[3] = ZigZag[6]<<2;

	DeZigZag[8] = ZigZag[2]<<1;
	DeZigZag[9] = ZigZag[4]<<1;
	DeZigZag[10] = ZigZag[7]<<1;
	DeZigZag[11] = ZigZag[13]<<2;

	DeZigZag[16] = ZigZag[3]<<1;
	DeZigZag[17] = ZigZag[8]<<1;
	DeZigZag[18] = ZigZag[12]<<2;
	DeZigZag[19] = ZigZag[17]<<2;

	DeZigZag[24] = ZigZag[9]<<2;
	DeZigZag[25] = ZigZag[11]<<2;
	DeZigZag[26] = ZigZag[18]<<2;
	DeZigZag[27] = ZigZag[24]<<3;
	}
	else 
	{
	DeZigZag[0] = ZigZag[0];
	DeZigZag[1] = ZigZag[1]<<2;
	DeZigZag[2] = ZigZag[5]<<2;
	DeZigZag[3] = ZigZag[6]<<3;

	DeZigZag[8] = ZigZag[2]<<2;
	DeZigZag[9] = ZigZag[4]<<2;
	DeZigZag[10] = ZigZag[7]<<2;
	DeZigZag[11] = ZigZag[13]<<4;

	DeZigZag[16] = ZigZag[3]<<2;
	DeZigZag[17] = ZigZag[8]<<2;
	DeZigZag[18] = ZigZag[12]<<3;
	DeZigZag[19] = ZigZag[17]<<4;

	DeZigZag[24] = ZigZag[9]<<3;
	DeZigZag[25] = ZigZag[11]<<4;
	DeZigZag[26] = ZigZag[18]<<4;
	DeZigZag[27] = ZigZag[24]<<4;
	}
#endif 

#if 0
	// Dequante
	if(YUVFlag == 1)
	{
		// Dequante Y
		for(i=0; i<64; i++)
		{
			ZigZag[i] = DeZigZag[i]<<DQTY[i];
		}
	}
	else
	{
		// Dequante UV
		for(i=0; i<64; i++)
		{
			ZigZag[i] = DeZigZag[i]<<DQTUV[i];
		}
	}
#endif 
#if 1
	// =============
	// == IDCT 1D ==
	// =============


//	for(i=0;i<8; i++)
//	{
//		for(j=0; j<4; j++)
//		{
			tmp1=a*DeZigZag[0]+c*DeZigZag[2];
			tmp2=b*DeZigZag[1];
			tmp3=d*DeZigZag[3];

			tmp=tmp1+tmp2+tmp3;
			temp[0] = tmp>>15;

			tmp=tmp1-tmp2-tmp3;
			temp[56] = tmp>>15;

			tmp1=a*DeZigZag[8]+c*DeZigZag[10];
			tmp2=b*DeZigZag[9];
			tmp3=d*DeZigZag[11];

			tmp=tmp1+tmp2+tmp3;
			temp[1] = tmp>>15;

			tmp=tmp1-tmp2-tmp3;
			temp[57] = tmp>>15;

			tmp1=a*DeZigZag[16]+c*DeZigZag[18];
			tmp2=b*DeZigZag[17];
			tmp3=d*DeZigZag[19];

			tmp=tmp1+tmp2+tmp3;
			temp[2] = tmp>>15;

			tmp=tmp1-tmp2-tmp3;
			temp[58] = tmp>>15;

			tmp1=a*DeZigZag[24]+c*DeZigZag[26];
			tmp2=b*DeZigZag[25];
			tmp3=d*DeZigZag[27];

			tmp=tmp1+tmp2+tmp3;
			temp[3] = tmp>>15;

			tmp=tmp1-tmp2-tmp3;
			temp[59] = tmp>>15;

//		for(j=0; j<4; j++)
//		{
			tmp1=a*DeZigZag[0]+f*DeZigZag[2];
			tmp2=d*DeZigZag[1];
			tmp3=g*DeZigZag[3];

			tmp=tmp1+tmp2-tmp3;
			temp[8] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[48] = tmp>>15;

			tmp1=a*DeZigZag[8]+f*DeZigZag[10];
			tmp2=d*DeZigZag[9];
			tmp3=g*DeZigZag[11];

			tmp=tmp1+tmp2-tmp3;
			temp[9] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[49] = tmp>>15;

			tmp1=a*DeZigZag[16]+f*DeZigZag[18];
			tmp2=d*DeZigZag[17];
			tmp3=g*DeZigZag[19];

			tmp=tmp1+tmp2-tmp3;
			temp[10] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[50] = tmp>>15;

			tmp1=a*DeZigZag[24]+f*DeZigZag[26];
			tmp2=d*DeZigZag[25];
			tmp3=g*DeZigZag[27];

			tmp=tmp1+tmp2-tmp3;
			temp[11] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[51] = tmp>>15;
//		}

//		for(j=0; j<4; j++)
//		{
			tmp1=a*DeZigZag[0]-f*DeZigZag[2];
			tmp2=e*DeZigZag[1];
			tmp3=b*DeZigZag[3];

			tmp=tmp1+tmp2-tmp3;
			temp[16] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[40] = tmp>>15;

			tmp1=a*DeZigZag[8]-f*DeZigZag[10];
			tmp2=e*DeZigZag[9];
			tmp3=b*DeZigZag[11];

			tmp=tmp1+tmp2-tmp3;
			temp[17] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[41] = tmp>>15;

			tmp1=a*DeZigZag[16]-f*DeZigZag[18];
			tmp2=e*DeZigZag[17];
			tmp3=b*DeZigZag[19];

			tmp=tmp1+tmp2-tmp3;
			temp[18] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[42] = tmp>>15;

			tmp1=a*DeZigZag[24]-f*DeZigZag[26];
			tmp2=e*DeZigZag[25];
			tmp3=b*DeZigZag[27];

			tmp=tmp1+tmp2-tmp3;
			temp[19] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[43] = tmp>>15;
//		}

//		for(j=0; j<4; j++)
//		{
			tmp1=a*DeZigZag[0]-c*DeZigZag[2];
			tmp2=g*DeZigZag[1];
			tmp3=e*DeZigZag[3];

			tmp=tmp1+tmp2-tmp3;
			temp[24] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[32] = tmp>>15;

			tmp1=a*DeZigZag[8]-c*DeZigZag[10];
			tmp2=g*DeZigZag[9];
			tmp3=e*DeZigZag[11];

			tmp=tmp1+tmp2-tmp3;
			temp[25] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[33] = tmp>>15;

			tmp1=a*DeZigZag[16]-c*DeZigZag[18];
			tmp2=g*DeZigZag[17];
			tmp3=e*DeZigZag[19];

			tmp=tmp1+tmp2-tmp3;
			temp[26] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[34] = tmp>>15;

			tmp1=a*DeZigZag[24]-c*DeZigZag[26];
			tmp2=g*DeZigZag[25];
			tmp3=e*DeZigZag[27];

			tmp=tmp1+tmp2-tmp3;
			temp[27] = tmp>>15;

			tmp=tmp1-tmp2+tmp3;
			temp[35] = tmp>>15;
//		}
		
//		for(j=0; j<4; j++)
//		{
/*			
			tmp = 0;
			for(k=0; k<4; k++)
			{
				tmp = tmp + coeft[32+k]*DeZigZag[8*j + k];
			}
			temp[32+j] = tmp>>15;
*/	
//		}

//		for(j=0; j<4; j++)
//		{
			
/*			tmp=a*DeZigZag[0] - e*DeZigZag[1] - f*DeZigZag[2] + b*DeZigZag[3];
			temp[40] = tmp>>15;
			
			tmp=a*DeZigZag[8] - e*DeZigZag[9] - f*DeZigZag[10] + b*DeZigZag[11];
			temp[41] = tmp>>15;
			
			tmp=a*DeZigZag[16] - e*DeZigZag[17] - f*DeZigZag[18] + b*DeZigZag[19];
			temp[42] = tmp>>15;
			
			tmp=a*DeZigZag[24] - e*DeZigZag[25] - f*DeZigZag[26] + b*DeZigZag[27];
			temp[43] = tmp>>15;
*//*
			tmp = 0;
			for(k=0; k<4; k++)
			{
				tmp = tmp + coeft[40 + k]*DeZigZag[8*j + k];
			}
			temp[40+j] = tmp>>15;
*/
//		}

//		for(j=0; j<4; j++)
//		{
			
/*			tmp=a*DeZigZag[0] - d*DeZigZag[1]+ f*DeZigZag[2] + g*DeZigZag[3];
			temp[48] = tmp>>15;
			
			tmp=a*DeZigZag[8] - d*DeZigZag[9] + f*DeZigZag[10] + g*DeZigZag[11];
			temp[49] = tmp>>15;
			
			tmp=a*DeZigZag[16] - d*DeZigZag[17] + f*DeZigZag[18] + g*DeZigZag[19];
			temp[50] = tmp>>15;
			
			tmp=a*DeZigZag[24] - d*DeZigZag[25] + f*DeZigZag[26] + g*DeZigZag[27];
			temp[51] = tmp>>15;
*/
//		}

//		for(j=0; j<4; j++)
//		{
			
/*			
			tmp = a*DeZigZag[0] - b*DeZigZag[1]+ c*DeZigZag[2] - d*DeZigZag[3];
			temp[56] = tmp>>15;
			
			tmp = a*DeZigZag[8] - b*DeZigZag[9]+ c*DeZigZag[10] - d*DeZigZag[11];
			temp[57] = tmp>>15;
			
			tmp = a*DeZigZag[16] - b*DeZigZag[17]+ c*DeZigZag[18] - d*DeZigZag[19];
			temp[58] = tmp>>15;
			
			tmp = a*DeZigZag[24] - b*DeZigZag[25]+ c*DeZigZag[26] - d*DeZigZag[27];
			temp[59] = tmp>>15;
*/

//		}


//	}

#endif 
	// =============
	// == IDCT 2D ==
	// =============
#if 1
//	for(i=0; i<8; i++)
//	{
			IPTRR_YUV = IPTRR;

//      i=0	
//		for(j=0; j<8; j++)
//		{ 
			// j=0
			tmp = temp[0]*a+temp[1]*b+temp[2]*c+temp[3]*d;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[0+IPTRR] = (BYTE) tmp;		

			// j=1
			tmp = temp[0]*a+temp[1]*d+temp[2]*f+temp[3]*(-g);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[1+IPTRR] = (BYTE) tmp;		

			// j=2
			tmp = temp[0]*a+temp[1]*e+temp[2]*(-f)+temp[3]*(-b);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[2+IPTRR] = (BYTE) tmp;		

			// j=3
			tmp = temp[0]*a+temp[1]*g+temp[2]*(-c)+temp[3]*(-e);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[3+IPTRR] = (BYTE) tmp;	

			// j=4
			tmp = temp[0]*a+temp[1]*(-g)+temp[2]*(-c)+temp[3]*e;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[4+IPTRR] = (BYTE) tmp;		

			// j=5
			tmp = temp[0]*a+temp[1]*(-e)+temp[2]*(-f)+temp[3]*b;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[5+IPTRR] = (BYTE) tmp;		

			// j=6
			tmp = temp[0]*a+temp[1]*(-d)+temp[2]*f+temp[3]*g;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[6+IPTRR] = (BYTE) tmp;		

			// j=7
			tmp = temp[0]*a+temp[1]*(-b)+temp[2]*c+temp[3]*(-d);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[7+IPTRR] = (BYTE) tmp;		
//		}

//		i=1
//		for(j=0; j<8; j++)
//		{ 
			IPTRR_YUV = X_WIDE+IPTRR_YUV;
			// j=0
			tmp = temp[8]*a+temp[9]*b+temp[10]*c+temp[11]*d;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV] = (BYTE) tmp;		

			// j=1
			tmp = temp[8]*a+temp[9]*d+temp[10]*f+temp[11]*(-g);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+1] = (BYTE) tmp;

			// j=2
			tmp = temp[8]*a+temp[9]*e+temp[10]*(-f)+temp[11]*(-b);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+2] = (BYTE) tmp;

			// j=3
			tmp = temp[8]*a+temp[9]*g+temp[10]*(-c)+temp[11]*(-e);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+3] = (BYTE) tmp;		

			// j=4
			tmp = temp[8]*a+temp[9]*(-g)+temp[10]*(-c)+temp[11]*e;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+4] = (BYTE) tmp;		

			// j=5
			tmp = temp[8]*a+temp[9]*(-e)+temp[10]*(-f)+temp[11]*b;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+5] = (BYTE) tmp;		

			// j=6
			tmp = temp[8]*a+temp[9]*(-d)+temp[10]*f+temp[11]*g;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+6] = (BYTE) tmp;		

			// j=7
			tmp = temp[8]*a+temp[9]*(-b)+temp[10]*c+temp[11]*(-d);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+7] = (BYTE) tmp;		
//		}

		IPTRR_YUV = X_WIDE+IPTRR_YUV;

//		i=2
//		for(j=0; j<8; j++)
//		{ 
			// j=0
			tmp = temp[16]*a+temp[17]*b+temp[18]*c+temp[19]*d;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV] = (BYTE) tmp;		

			// j=1
			tmp = temp[16]*a+temp[17]*d+temp[18]*f+temp[19]*(-g);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+1] = (BYTE) tmp;

			// j=2
			tmp = temp[16]*a+temp[17]*e+temp[18]*(-f)+temp[19]*(-b);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+2] = (BYTE) tmp;

			// j=3
			tmp = temp[16]*a+temp[17]*g+temp[18]*(-c)+temp[19]*(-e);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+3] = (BYTE) tmp;		

			// j=4
			tmp = temp[16]*a+temp[17]*(-g)+temp[18]*(-c)+temp[19]*e;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+4] = (BYTE) tmp;		

			// j=5
			tmp = temp[16]*a+temp[17]*(-e)+temp[18]*(-f)+temp[19]*b;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+5] = (BYTE) tmp;		

			// j=6
			tmp = temp[16]*a+temp[17]*(-d)+temp[18]*f+temp[19]*g;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+6] = (BYTE) tmp;		

			// j=7
			tmp = temp[16]*a+temp[17]*(-b)+temp[18]*c+temp[19]*(-d);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+7] = (BYTE) tmp;		
//		}
		IPTRR_YUV = X_WIDE+IPTRR_YUV;
//		i=3
//		for(j=0; j<8; j++)
//		{ 
			// j=0
			tmp = temp[24]*a+temp[25]*b+temp[26]*c+temp[27]*d;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV] = (BYTE) tmp;		

			// j=1
			tmp = temp[24]*a+temp[25]*d+temp[26]*f+temp[27]*(-g);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+1] = (BYTE) tmp;

			// j=2
			tmp = temp[24]*a+temp[25]*e+temp[26]*(-f)+temp[27]*(-b);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+2] = (BYTE) tmp;

			// j=3
			tmp = temp[24]*a+temp[25]*g+temp[26]*(-c)+temp[27]*(-e);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+3] = (BYTE) tmp;		

			// j=4
			tmp = temp[24]*a+temp[25]*(-g)+temp[26]*(-c)+temp[27]*e;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+4] = (BYTE) tmp;		

			// j=5
			tmp = temp[24]*a+temp[25]*(-e)+temp[26]*(-f)+temp[27]*b;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+5] = (BYTE) tmp;		

			// j=6
			tmp = temp[24]*a+temp[25]*(-d)+temp[26]*f+temp[27]*g;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+6] = (BYTE) tmp;		

			// j=7
			tmp = temp[24]*a+temp[25]*(-b)+temp[26]*c+temp[27]*(-d);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+7] = (BYTE) tmp;		
//		}
		IPTRR_YUV = X_WIDE+IPTRR_YUV;
//		i=4
//		for(j=0; j<8; j++)
//		{ 
			// j=0
			tmp = temp[32]*a+temp[33]*b+temp[34]*c+temp[35]*d;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV] = (BYTE) tmp;		

			// j=1
			tmp = temp[32]*a+temp[33]*d+temp[34]*f+temp[35]*(-g);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+1] = (BYTE) tmp;

			// j=2
			tmp = temp[32]*a+temp[33]*e+temp[34]*(-f)+temp[35]*(-b);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+2] = (BYTE) tmp;

			// j=3
			tmp = temp[32]*a+temp[33]*g+temp[34]*(-c)+temp[35]*(-e);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+3] = (BYTE) tmp;		

			// j=4
			tmp = temp[32]*a+temp[33]*(-g)+temp[34]*(-c)+temp[35]*e;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+4] = (BYTE) tmp;		

			// j=5
			tmp = temp[32]*a+temp[33]*(-e)+temp[34]*(-f)+temp[35]*b;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+5] = (BYTE) tmp;		

			// j=6
			tmp = temp[32]*a+temp[33]*(-d)+temp[34]*f+temp[35]*g;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+6] = (BYTE) tmp;		

			// j=7
			tmp = temp[32]*a+temp[33]*(-b)+temp[34]*c+temp[35]*(-d);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+7] = (BYTE) tmp;		
//		}
		IPTRR_YUV = X_WIDE+IPTRR_YUV;
//		i=5
//		for(j=0; j<8; j++)
//		{ 
			// j=0
			tmp = temp[40]*a+temp[41]*b+temp[42]*c+temp[43]*d;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV] = (BYTE) tmp;		

			// j=1
			tmp = temp[40]*a+temp[41]*d+temp[42]*f+temp[43]*(-g);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+1] = (BYTE) tmp;

			// j=2
			tmp = temp[40]*a+temp[41]*e+temp[42]*(-f)+temp[43]*(-b);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+2] = (BYTE) tmp;

			// j=3
			tmp = temp[40]*a+temp[41]*g+temp[42]*(-c)+temp[43]*(-e);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+3] = (BYTE) tmp;		

			// j=4
			tmp = temp[40]*a+temp[41]*(-g)+temp[42]*(-c)+temp[43]*e;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+4] = (BYTE) tmp;		

			// j=5
			tmp = temp[40]*a+temp[41]*(-e)+temp[42]*(-f)+temp[43]*b;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+5] = (BYTE) tmp;		

			// j=6
			tmp = temp[40]*a+temp[41]*(-d)+temp[42]*f+temp[43]*g;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+6] = (BYTE) tmp;		

			// j=7
			tmp = temp[40]*a+temp[41]*(-b)+temp[42]*c+temp[43]*(-d);
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+7] = (BYTE) tmp;		

//		}
		IPTRR_YUV = X_WIDE+IPTRR_YUV;
//		i=6
//		for(j=0; j<8; j++)
//		{ 
			tmp1=temp[48]*a+temp[50]*c;
			tmp2=temp[49]*b;
			tmp3=temp[51]*d;
			// j=0
			tmp=tmp1+tmp2+tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV] = (BYTE) tmp;		

			// j=7
			tmp=tmp1-tmp2-tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+7] = (BYTE) tmp;		

			tmp1=temp[48]*a+temp[50]*f;
			tmp2=temp[49]*d;
			tmp3=temp[51]*g;
			// j=1
			tmp=tmp1+tmp2-tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+1] = (BYTE) tmp;

			// j=6
			tmp=tmp1-tmp2+tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+6] = (BYTE) tmp;		
			
			tmp1=temp[48]*a-temp[50]*f;
			tmp2=temp[49]*e;
			tmp3=temp[51]*b;
			// j=2
			tmp=tmp1+tmp2-tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+2] = (BYTE) tmp;

			// j=5
			tmp=tmp1-tmp2+tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+5] = (BYTE) tmp;		

			tmp1=temp[48]*a-temp[50]*c;
			tmp2=temp[49]*g;
			tmp3=temp[51]*e;
			// j=3
			tmp=tmp1+tmp2-tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+3] = (BYTE) tmp;		

			// j=4
			tmp=tmp1-tmp2+tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+4] = (BYTE) tmp;		
//		}
		IPTRR_YUV = X_WIDE+IPTRR_YUV;
//		i=7
//		for(j=0; j<8; j++)
//		{ 
			tmp1=temp[56]*a+temp[58]*c;
			tmp2=temp[57]*b;
			tmp3=temp[59]*d;

			// j=0
			tmp=tmp1+tmp2+tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV] = (BYTE) tmp;		

			// j=7
			tmp=tmp1-tmp2-tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+7] = (BYTE) tmp;		

			tmp1=temp[56]*a+temp[58]*f;
			tmp2=temp[57]*d;
			tmp3=temp[59]*g;

			// j=1
			tmp=tmp1+tmp2-tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+1] = (BYTE) tmp;

			// j=6
			tmp=tmp1-tmp2+tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+6] = (BYTE) tmp;		

			tmp1=temp[56]*a-temp[58]*f;
			tmp2=temp[57]*e;
			tmp3=temp[59]*b;

			// j=2
			tmp=tmp1+tmp2-tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+2] = (BYTE) tmp;

			// j=5
			tmp=tmp1-tmp2+tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+5] = (BYTE) tmp;		

			tmp1=temp[56]*a-temp[58]*c;
			tmp2=temp[57]*g;
			tmp3=temp[59]*e;

			// j=3
			tmp=tmp1+tmp2-tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+3] = (BYTE) tmp;		

			// j=4
			tmp=tmp1-tmp2+tmp3;
			tmp = tmp>>15;
			tmp=tmp+128;
			if(tmp >255) tmp=255;
			if(tmp < 0 ) tmp=0;
			lpDestBuf[IPTRR_YUV+4] = (BYTE) tmp;		

//		}

	
//	}

#endif 
	TempInfo->index=index;
	TempInfo->IPTRR_U=IPTRR+8;

	return 0;
} 

static NTSTATUS 
Decompress420HiNoMMX(
	PUCHAR  pY,
	ULONG   FrameLength,
	ULONG   X_SIZE, 
	ULONG   Y_SIZE, 
	PUCHAR	TableBuffer,
	PUCHAR	TableUVBuffer,
	PSHORT  CoefBuffer,
	PULONG  ZigZagTable,
	PUCHAR  RawFrameBuffer,
	ULONG   ProcessedDataLength,	
	PUCHAR  TempBuffer,
	CompInfo *TempInfo)

{

// Begin of decoder definition 
//POVTCAM_DEVICE_CONTEXT deviceContext = DeviceContext;
LPBYTE lpYUV, lpU, lpV, lpTMPBUF;
ULONG	PTRX, PTRY;
// ULONG i;
ULONG	IPTRR, IPTRR_U, IPTRR_V;
//ULONG   ipm, tms, Tcnt=1600;
ULONG   index;
ULONG   SrcUVDataLength;
//ULONG   SrcYDataLength;
//short  a,b,c,d,e,mid;

PUCHAR  TempBuffU = NULL; 
PUCHAR  TempBuffV = NULL; 
PUCHAR  TempBuffY = NULL; 

DWORDLONG   Y_Scalor=0x7fff7fff7fff7fff;

//   u=0.564
DWORDLONG   U_Scalor=0x4831483148314831;
//   v=0.713
DWORDLONG   V_Scalor=0x5b435b435b435b43;

//DWORDLONG   U_Scalor=0x7fff7fff7fff7fff;
//DWORDLONG   V_Scalor=0x7fff7fff7fff7fff;
//DWORDLONG   CONST128=0x8080808080808080;
//DWORDLONG   CONST128=0x0080008000800080; /* Previously disabled by OVT */
//DWORDLONG   Const4000=0x4000400040004000;


// 	TempBuffer = deviceContext->TempBuffer;
//	TableBuffer = deviceContext->TableBuffer;  //TempBuffer;
//	CoefBuffer = deviceContext->CoefBuffer;  //TempBuffer;
 //	ZigZagTable = deviceContext->ZigZagTable;
 	lpTMPBUF = RawFrameBuffer; // + 9;  //TempBuffer;

//ProcessedDataLength = deviceContext->ProcessedDataLength;
//ProcessedDataLength = ProcessedDataLength/8;
//ProcessedDataLength = ProcessedDataLength*8 + 8;

SrcUVDataLength = X_SIZE  * Y_SIZE/4;
//SrcYDataLength = X_SIZE  * Y_SIZE;

TempBuffU= TempBuffer;
TempBuffV= TempBuffU + SrcUVDataLength;
TempBuffY= TempBuffV + SrcUVDataLength;

		   lpYUV = pY;
		     lpU = lpYUV+X_SIZE*Y_SIZE;
		     lpV = lpU + X_SIZE*Y_SIZE/4;

index=0;
// ===== 320x240 images    
 for (PTRY=0; PTRY<Y_SIZE; PTRY=PTRY+16)
 {// start Y Loop

IPTRR=X_SIZE*PTRY;
IPTRR_U=(IPTRR)>>2;
IPTRR_V=IPTRR_U; 

 if (index > ProcessedDataLength)
	return STATUS_UNSUCCESSFUL;

 for (PTRX=0; PTRX<X_SIZE; PTRX=PTRX+32)
	{ // start X loop
		// ##############################
		// # Decompression              #
		// ##############################
	     /* lpDestBuf, lpTMPBUF, index, IPTRR, X_WIDE, TableBufferYUV, IDCT_Coef, ZZLN, UV_Scalor, YUVFlag */
//DecompressYHI(lpU, lpTMPBUF, index, IPTRR_U, X_SIZE/2,  TableBuffer+64, CoefBuffer, ZigZagTable, U_Scalor, 2, TempInfo);
DecompressYHINoMMX(lpU, lpTMPBUF, index, IPTRR_U, X_SIZE/2,  TableUVBuffer, CoefBuffer, ZigZagTable, U_Scalor, 2, TempInfo);
index=TempInfo->index;
IPTRR_U=TempInfo->IPTRR_U;

//DecompressYHINoMMX(lpV, lpTMPBUF, index, IPTRR_V, X_SIZE/2, TableBuffer+64, CoefBuffer, ZigZagTable, V_Scalor, 2, TempInfo);
DecompressYHINoMMX(lpV, lpTMPBUF, index, IPTRR_V, X_SIZE/2, TableUVBuffer, CoefBuffer, ZigZagTable, V_Scalor, 2, TempInfo);
index=TempInfo->index;
IPTRR_V=TempInfo->IPTRR_U;

DecompressYHINoMMX(lpYUV, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
//DecompressYHINoMMX(TempBuffY, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
index=TempInfo->index;
IPTRR=TempInfo->IPTRR_U;

DecompressYHINoMMX(lpYUV, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
//DecompressYHINoMMX(TempBuffY, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
index=TempInfo->index;
IPTRR=TempInfo->IPTRR_U;
 
DecompressYHINoMMX(lpYUV, lpTMPBUF, index, IPTRR, X_SIZE,  TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
//DecompressYHINoMMX(TempBuffY, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
index=TempInfo->index;
IPTRR=TempInfo->IPTRR_U;

DecompressYHINoMMX(lpYUV, lpTMPBUF, index, IPTRR, X_SIZE,  TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
//DecompressYHINoMMX(TempBuffY, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
index=TempInfo->index;
IPTRR=TempInfo->IPTRR_U;
	 } //Stop X


IPTRR=X_SIZE*(PTRY+8);
IPTRR_V=(X_SIZE*PTRY + X_SIZE); // /4;
IPTRR_V=IPTRR_V>>2;
IPTRR_U=IPTRR_V;
 
 for (PTRX=0; PTRX<X_SIZE; PTRX=PTRX+32)
	{ // start X loop
		// ##############################
		// # Decompression				#
		// ##############################

//DecompressYHI(lpU, lpTMPBUF, index, IPTRR_U, X_SIZE/2, TableBuffer+64,CoefBuffer, ZigZagTable, U_Scalor, 2, TempInfo);
DecompressYHINoMMX(lpU, lpTMPBUF, index, IPTRR_U, X_SIZE/2,  TableUVBuffer, CoefBuffer, ZigZagTable, U_Scalor, 2, TempInfo);
index=TempInfo->index;
IPTRR_U=TempInfo->IPTRR_U;

//DecompressYHINoMMX(lpV, lpTMPBUF, index, IPTRR_V, X_SIZE/2, TableBuffer+64, CoefBuffer, ZigZagTable, V_Scalor, 2, TempInfo);
DecompressYHINoMMX(lpV, lpTMPBUF, index, IPTRR_V, X_SIZE/2, TableUVBuffer, CoefBuffer, ZigZagTable, V_Scalor, 2, TempInfo);
index=TempInfo->index;
IPTRR_V=TempInfo->IPTRR_U;

DecompressYHINoMMX(lpYUV, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
//DecompressYHINoMMX(TempBuffY, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
index=TempInfo->index;
IPTRR=TempInfo->IPTRR_U;

DecompressYHINoMMX(lpYUV, lpTMPBUF, index, IPTRR, X_SIZE,  TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
//DecompressYHINoMMX(TempBuffY, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
index=TempInfo->index;
IPTRR=TempInfo->IPTRR_U;
 
DecompressYHINoMMX(lpYUV, lpTMPBUF, index, IPTRR, X_SIZE,  TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
//DecompressYHINoMMX(TempBuffY, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
index=TempInfo->index;
IPTRR=TempInfo->IPTRR_U;

DecompressYHINoMMX(lpYUV, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
//DecompressYHINoMMX(TempBuffY, lpTMPBUF, index, IPTRR, X_SIZE, TableBuffer, CoefBuffer, ZigZagTable, Y_Scalor, 1, TempInfo);
index=TempInfo->index;
IPTRR=TempInfo->IPTRR_U;
	 } //Stop X
}// stop Y


	return 0;
}




/* Input (RawFrameBuffer) format is raw isoc. data (with header and packet 
 * nunber stripped).
 * Output (FrameBuffer) format is planar YUV420
 * Returns uncompressed data length if success, or zero if error
 */
static ULONG 
Decompress420(	ULONG SrcWidth,
		ULONG SrcHeight,
		PVOID FrameBuffer,
		ULONG FrameLength,
		PVOID RawFrameBuffer,
		ULONG RawFrameLength,
		ULONG ActualRawFrameLength,
		struct ov511_frame *frame)
{
	ULONG dwLength = SrcWidth * SrcHeight;
	PUCHAR pBuffer;
//	PUCHAR pOutBuffer;
	PUCHAR pYQuanTable;
	PUCHAR pUVQuanTable;
	PSHORT pIDCTTable;
	PULONG pZigAzgTable;
	NTSTATUS ntStatus = STATUS_SUCCESS;
//	UCHAR YTable[QUANTABLESIZE], UVTable[QUANTABLESIZE];
//	UCHAR Temp;
//	ULONG i;

#if 0
	PDEBUG(4, "w=%lu h=%lu fb=%p fblen=%lu rawfb=%p rawlen=%lu, actlen=%lu",
		SrcWidth, SrcHeight, FrameBuffer, FrameLength, RawFrameBuffer,
		RawFrameLength, ActualRawFrameLength);
#endif

//#ifdef OV518
//	pgFrameContext = m_pCurrentFrameContext;
////	OVTCAM_KdPrint(MIN_TRACE, ("OVTCAM: YUV=420 \n")); 
//#endif
	if(frame->m_pTempBuffer && frame->m_TempBufferLength < dwLength)
	{
		FREE(frame->m_pTempBuffer);
		frame->m_pTempBuffer = NULL;
		frame->m_TempBufferLength = 0;
	}
	if(frame->m_pTempBuffer == NULL)
	{
		frame->m_pTempBuffer = ALLOCATE(NonPagedPool, dwLength,0);
		frame->m_TempBufferLength = dwLength ;
	}
	if(frame->m_pTempBuffer == NULL)
		return 0;

	pBuffer = (PUCHAR)frame->m_pTempBuffer;
//	pOutBuffer = (PUCHAR)FrameBuffer;
	pYQuanTable = GetYQuanTable();
	pUVQuanTable = GetUVQuanTable();
	pIDCTTable = GetIDCTTable();
	pZigAzgTable = GetZigZagTable();

//#if 0
//	//def OV518
//		for(int i = 0; i < QUANTABLESIZE; i++)
//		{
//			Temp = *pYQuanTable++;
//			if(Temp > 4) YTable[i] = 3;
//			else YTable[i] = Temp - 2;
//
//			Temp = *pUVQuanTable++;
//			if(Temp > 4) UVTable[i] = 3;
//			else UVTable[i] = Temp - 2;
//		}
//#endif

//	if(m_bSupportMMX)
//	{
//#ifdef OV518
//
//#ifdef  TABLE_DBG
//		ntStatus = Decompress420OV518NoMMXTABLE((PUCHAR) FrameBuffer, FrameLength, SrcWidth, SrcHeight, (PUCHAR)pYQuanTable, (PUCHAR)pUVQuanTable,pIDCTTable,pZigAzgTable,(PUCHAR) RawFrameBuffer,ActualRawFrameLength,pBuffer); 
//#else
//		ntStatus = Decompress420OV518((PUCHAR) FrameBuffer, FrameLength, SrcWidth, SrcHeight, (PUCHAR)YTable, (PUCHAR)UVTable,pIDCTTable,pZigAzgTable,(PUCHAR) RawFrameBuffer,ActualRawFrameLength,pBuffer); 
//#endif
//
//#else
//		ntStatus = Decompress420Hi((PUCHAR) FrameBuffer, FrameLength, SrcWidth, SrcHeight, pYQuanTable, pUVQuanTable,pIDCTTable,pZigAzgTable,(PUCHAR) RawFrameBuffer,ActualRawFrameLength,pBuffer);
//#endif
//	}
//	else
//	{

//#ifdef OV518
//#ifdef  TABLE_DBG
//		ntStatus = Decompress420OV518NoMMXTABLE((PUCHAR) FrameBuffer, FrameLength, SrcWidth, SrcHeight, (PUCHAR)pYQuanTable, (PUCHAR)pUVQuanTable,pIDCTTable,pZigAzgTable,(PUCHAR) RawFrameBuffer,ActualRawFrameLength,pBuffer); 
//#else
//		ntStatus = Decompress420OV518NoMMX((PUCHAR) FrameBuffer, FrameLength, SrcWidth, SrcHeight, (PUCHAR)YTable, (PUCHAR)UVTable,pIDCTTable,pZigAzgTable,(PUCHAR) RawFrameBuffer,ActualRawFrameLength,pBuffer); 
//#endif
//#else
		ntStatus = Decompress420HiNoMMX((PUCHAR) FrameBuffer, FrameLength, SrcWidth, SrcHeight, pYQuanTable, pUVQuanTable,pIDCTTable,pZigAzgTable,(PUCHAR) RawFrameBuffer,ActualRawFrameLength,pBuffer,&(frame->TempInfo));
//#endif

// Disabled by Mark
//		//for color saturation 8/22/00
//		{
//			LONG u , v;
//			PUCHAR pU = (PUCHAR) FrameBuffer + dwLength;
//			PUCHAR pV = pU + dwLength/4;
//			for(ULONG i = 0; i < dwLength/4; i++)
//			{
//				u = (LONG)*pU;
//				v = (LONG)*pV;
//				u -= 128;
//				v -= 128;
//				u = (u * 577) >> 10;
//				v = (v * 730) >> 10;
//				u += 128;
//				v += 128;
//				*pU++ = (UCHAR)u;
//				*pV++ = (UCHAR)v;
////   u=0.564
////   v=0.713
//			}
//		}


//	}

	// Added by Steven 11/06/2000
	if(ntStatus) return 0;

//	if(m_EnColorCorrect)
//	{
//		ColorCorrect420((PUCHAR) FrameBuffer, FrameLength, SrcWidth, SrcHeight);
//	}

//	OVTCAM_CopyMemory(pOutBuffer, RawFrameBuffer, dwLength * 3 / 2);
//	if(!NT_SUCCESS(ntStatus))
//		return 0;
	return dwLength * 3 / 2;
}

static void DecompLock(void)
{
	MOD_INC_USE_COUNT;
}

static void DecompUnlock(void)
{
	MOD_DEC_USE_COUNT;
}

static struct ov51x_decomp_ops decomp_ops = {
	decomp_420:	Decompress420,	
	decomp_lock:	DecompLock,
	decomp_unlock:	DecompUnlock
};

static int __init decomp_init(void)
{
	int rc;

	rc = ov511_register_decomp_module(DECOMP_INTERFACE_VER, &decomp_ops, ov518, mmx);
	if (rc) {
//		err("Could not register with ov511 (rc=%d)", rc);
		return -1;
	}

//	info("version %s registered", version);

	return 0;
}

static void __exit decomp_exit(void)
{
	ov511_deregister_decomp_module(ov518, mmx);
//	info("deregistered\n");
}

module_init(decomp_init);
module_exit(decomp_exit);
